/*
 * Copyright (c) 1998, 2007 Sun Microsystems, Inc. 
 * All  Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * -Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * -Redistribution in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES OR LIABILITIES SUFFERED BY LICENSEE
 * AS A RESULT OF OR RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE
 * LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
 * SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED
 * AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 * 
 * You acknowledge that this software is not designed,licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility.
 */

package com.sun.xfilechooser;

import java.util.ListResourceBundle;

/**
 * EditorResource class is needed for I8N. It defines
 * the strings that should be internationalized.
 */
public class EditorResource extends ListResourceBundle {
    public Object[][] getContents() {
	return contents;
    }
    
    static final Object[][] contents = {
	{"Dialog Type", "Dialog Type"},
	{"Dialog Title", "Dialog Title"},
	{"File Selection Mode", "File Selection Mode"},
	{"Show Hidden Files", "Show Hidden Files"},
	{"Approve Button Text", "Approve Button Text"},
	{"Approve Button Tooltip", "Approve Button Tooltip"},
	{"Approve Button Mnemonic", "Approve Button Mnemonic"},
	{"Set Current Directory", "Set Current Directory"},
	{"Set Background Color", "Set Background Color"},
	{"Set Foreground Color", "Set Foreground Color"},	
	{"Open", "Open"},
	{"Save", "Save"},
	{"Custom", "Custom"},
	{"Files Only", "Files Only"},
	{"Directories Only", "Directories Only"},
	{"Files/Directories", "Files/Directories"},
	{"New Folder", "New Folder"},
	{"NewFolder", "NewFolder"}	
    };
}

    
	
	
	
	
	
	
	
	
	
	    
